import type { Context, CSSProperties, FunctionComponent, ReactElement, ReactNode, ReactText } from 'react';
declare const Edge: FunctionComponent<EdgeProps>;
declare const Edges: FunctionComponent<EdgesProps>;
declare const Graph: FunctionComponent<GraphProps>;
declare const Node: FunctionComponent<NodeProps>;
declare const Nodes: FunctionComponent<NodesProps>;
declare const Path: FunctionComponent<PathProps>;
declare const GraphContext: Context<GraphContextProps>;
declare const GraphGlobalSettingsContext: Context<GraphGlobalSettingsContextProps>;
declare class NodePositions {
    constructor(frequency?: number);
    getFrequency: () => number;
    setFrequency: (frequency: number) => number;
    subscribe: (id: string, callback_: (size: Size, nodes: NodesDimensions) => void) => void;
    unsubscribe: (id: string) => void;
    register: (node: Dimensions) => void;
    unregister: (id: ReactText) => void;
    update: (node: Dimensions) => void;
    match: (x: number, y: number) => Promise<ReactText>;
    size: () => Size;
}
declare class EdgeSubscriber {
    constructor(frequency?: number);
    getFrequency: () => number;
    setFrequency: (frequency: number) => number;
    register: <T>(edgeId: ReactText, nodeId: ReactText, updateFn: (update: T) => void) => EdgeNodeProps;
    unregister: (edgeId: ReactText, nodeId: ReactText) => void;
    update: (props: EdgeNodeProps) => void;
}
export { Edge, Edges, Graph, GraphContext, GraphGlobalSettingsContext, Node, Nodes, Path };
export declare type EdgeNodeProps = {
    height: number;
    id: ReactText;
    shape: 'oval' | 'rectangle';
    width: number;
    x: number;
    y: number;
};
export declare type Line = 'curve' | 'direct' | 'step';
export declare type Shape = 'oval' | 'rectangle';
export declare type EdgeProps = {
    children?: ReactNode;
    id: ReactText;
    line?: Line;
    markerEnd?: string;
    markerStart?: string;
    path?: (source: EdgeNodeProps, target: EdgeNodeProps, children: ReactNode) => ReactElement;
    sourceId: ReactText;
    targetId: ReactText;
};
export declare type EdgesProps = {
    children: Array<ReactElement<EdgeProps>> | ReactElement<EdgeProps>;
};
export declare type PathProps = {
    children?: ReactNode;
    diagramId?: ReactText;
    className?: string;
    id: ReactText;
    line?: Line;
    markerEnd?: string;
    markerStart?: string;
    source: EdgeNodeProps;
    target: EdgeNodeProps;
};
export declare type NodeProps = {
    children?: ReactElement | ReactElement[];
    className?: string;
    height?: number;
    id: ReactText;
    ref?: React.MutableRefObject<HTMLDivElement> | ((instance: HTMLDivElement) => void);
    shape?: Shape;
    style?: CSSProperties;
    width?: number;
    x?: number;
    y?: number;
};
export declare type NodesProps = {
    children: Array<ReactElement<NodeProps>> | ReactElement<NodeProps>;
};
export declare type GraphProps = {
    children?: Array<ReactElement<EdgesProps | NodesProps>>;
    edgeFrequency?: number;
    id: ReactText;
    line?: Line;
    nodeFrequency?: number;
    nodeHeight?: number;
    nodeWidth?: number;
    shape?: Shape;
};
export declare type Dimensions = {
    height: number;
    id: ReactText;
    width: number;
    x: number;
    y: number;
};
export declare type Size = {
    bottom: number;
    left: number;
    right: number;
    top: number;
};
export declare type NodesDimensions = {
    [key: string]: Dimensions;
};
export declare type GraphContextProps = {
    diagramId: ReactText;
    nodePositions: NodePositions;
    edgeSubscriber: EdgeSubscriber;
};
export declare type GraphGlobalSettingsContextProps = {
    height?: number;
    line?: Line;
    shape?: Shape;
    width?: number;
};
